import org.jboss.seam.Component;
import org.jboss.seam.mock.SeamTest;
import java.util.*;
import org.testng.annotations.Test;
import javax.persistence.*;

public class HelloWorldTest extends SeamTest {
   
  @Test
  public void testSayHello() throws Exception {
    
    new FacesRequest("/hello.xhtml") {
      
      @Override
      protected void updateModelValues() throws Exception {
        setValue("#{person.name}", "Michael Yuan");
      }

      @Override
      protected void invokeApplication() {
        assert getValue ("#{person.name}").equals("Michael Yuan");
        assert invokeMethod("#{manager.sayHello}").equals("fans");
      }

      @Override
      protected void renderResponse() {
        List<Person> fans = 
            (List<Person>) getValue("#{manager.fans}");
        assert fans!=null;
        assert fans.get(fans.size()-1)
                   .getName().equals("Michael Yuan");
        
        assert getValue("#{person.name}").equals("Michael Yuan");
      }

    }.run();
      
  }
  
  @Test
  public void unitTestStartOver() throws Exception {
    Manager manager = new ManagerAction ();
    assert manager.startOver().equals("hello");
    
    Person person = (Person) getField(manager, "person");
    assert person != null;
    assert person.getName() == null;
  }

  @Test
  public void simulateBijection() throws Exception {
    Manager manager = new ManagerAction ();
    Person in = new Person ();
    in.setName ("Michael Yuan");
    
    setField (manager, "person", in);
    Person out = (Person) getField(manager, "person");
    assert out != null;
    assert out.getName().equals("Michael Yuan");
    
    assert manager.startOver().equals("hello");
    
    out = (Person) getField(manager, "person");
    assert out != null;
    assert out.getName() == null;
  }
  
  @Test
  public void unitTestSayHello() throws Exception {

    Manager manager = new ManagerAction ();

    EntityManagerFactory emf = 
      Persistence.createEntityManagerFactory("helloworld");
    EntityManager em = emf.createEntityManager();
    setField(manager, "em", em);

    Person person = new Person ();
    person.setName ("Thomas Heute");
    setField(manager, "person", person);
    setField(manager, "confirmed", false);

    // getUserTransaction().begin();
    em.getTransaction().begin();
    String outcome = manager.sayHello ();
    em.getTransaction().commit();
    // getUserTransaction().commit();

    assert outcome.equals("fans");
    
    List <Person> fans = manager.getFans();
    assert fans!=null;
    assert fans.get(fans.size()-1)
               .getName().equals("Thomas Heute");

    person = (Person) getField (manager, "person");
    assert person != null;
    // assert greeter.getName() == null;
    assert person.getName().equals("Thomas Heute");
    
    em.close();
  }
  
  @Test
  public void unitTestSayHello2() throws Exception {

    Manager manager = new ManagerAction ();

    EntityManagerFactory emf = 
      Persistence.createEntityManagerFactory("helloworld");
    EntityManager em = emf.createEntityManager();
    setField(manager, "em", em);

    Person person = new Person ();
    person.setName ("Thomas");
    setField(manager, "person", person);
    setField(manager, "confirmed", false);

    // getUserTransaction().begin();
    em.getTransaction().begin();
    String outcome = manager.sayHello ();
    em.getTransaction().commit();
    // getUserTransaction().commit();

    assert outcome.equals("warning");
    
    List <Person> fans = 
        (List<Person>) getField (manager, "fans");
    assert fans==null;

    person = (Person) getField (manager, "person");
    assert person != null;
    assert person.getName().equals("Thomas");
    
    em.close();
  }
   
}
